# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Keras mixed precision API.

See [the mixed precision guide](
  https://www.tensorflow.org/guide/keras/mixed_precision) to learn how to
use the API.

"""

from __future__ import print_function as _print_function

import sys as _sys

from . import experimental
from tensorflow.python.keras.mixed_precision.loss_scale_optimizer import LossScaleOptimizer
from tensorflow.python.keras.mixed_precision.policy import Policy
from tensorflow.python.keras.mixed_precision.policy import global_policy
from tensorflow.python.keras.mixed_precision.policy import set_policy as set_global_policy

del _print_function
